/*
 * Copyright (c) 2008-2017 Haulmont.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.haulmont.cuba.gui.xml.layout.loaders;

import com.haulmont.cuba.gui.GuiDevelopmentException;
import com.haulmont.cuba.gui.components.Image;
import com.haulmont.cuba.gui.data.Datasource;
import org.apache.commons.lang3.StringUtils;
import org.dom4j.Element;

public class ImageLoader extends AbstractResourceViewLoader<Image> {

    @Override
    public void createComponent() {
        resultComponent = factory.create(Image.NAME);
        loadId(resultComponent, element);
    }

    @Override
    public void loadComponent() {
        super.loadComponent();

        loadContainer(resultComponent, element);
        if (resultComponent.getValueSource() == null) {
            loadDatasource(resultComponent, element);
        }

        loadScaleMode(resultComponent, element);
    }

    protected void loadScaleMode(Image image, Element element) {
        String scaleModeString = element.attributeValue("scaleMode");
        Image.ScaleMode scaleMode = Image.ScaleMode.NONE;
        if (scaleModeString != null) {
            scaleMode = Image.ScaleMode.valueOf(scaleModeString);
        }
        image.setScaleMode(scaleMode);
    }

    protected void loadDatasource(Image component, Element element) {
        final String datasource = element.attributeValue("datasource");
        if (!StringUtils.isEmpty(datasource)) {
            Datasource ds = getComponentContext().getDsContext().get(datasource);
            if (ds == null) {
                throw new GuiDevelopmentException(String.format("Datasource '%s' is not defined", datasource),
                        getContext(), "Component ID", component.getId());
            }
            String property = element.attributeValue("property");
            if (StringUtils.isEmpty(property)) {
                throw new GuiDevelopmentException(
                        String.format("Can't set datasource '%s' for component '%s' because 'property' " +
                                "attribute is not defined", datasource, component.getId()), context);
            }

            component.setDatasource(ds, property);
        }
    }
}